"""
Distutils doesn't support nasm, so this is a custom compiler for NASM
"""

from distutils.unixccompiler import UnixCCompiler
from distutils.sysconfig import get_config_var
import sys


class NasmCompiler(UnixCCompiler) :
    compiler_type = 'nasm'
    src_extensions = ['.asm']
    obj_extension = '.obj'
    language_map = {".asm"   : "asm",}
    language_order = ["asm"]
    executables = {'preprocessor' : None,
               'compiler'     : ["nasm"],
               'compiler_so'  : ["nasm"],
               'compiler_cxx' : ["nasm"],
               'linker_so'    : ["cc", "-shared"],
               'linker_exe'   : ["cc", "-shared"],
               'archiver'     : ["ar", "-cr"],
               'ranlib'       : None,
               }
    def __init__ (self,
                  verbose=0,
                  dry_run=0,
                  force=0):

        UnixCCompiler.__init__ (self, verbose, dry_run, force)
        self.set_executable("compiler", "nasm")

    def _is_gcc(self, compiler_name):
        return False  # ¯\_(ツ)_/¯

    def _get_cc_args(self, pp_opts, debug, before):
        if sys.platform == 'darwin':
            # Fix the symbols on macOS
            cc_args = pp_opts + ["-f macho64","-DNOPIE","--prefix=_"]
        else:
            # Use 64-bit elf format for Linux
            cc_args = pp_opts + ["-f elf64"]
        if debug:
            # Debug symbols from NASM
            cc_args[:0] = ['-g']
        if before:
            cc_args[:0] = before
        return cc_args

    def link(self, target_desc, objects,
             output_filename, output_dir=None, libraries=None,
             library_dirs=None, runtime_library_dirs=None,
             export_symbols=None, debug=0, extra_preargs=None,
             extra_postargs=None, build_temp=None, target_lang=None):
        # Make sure libpython gets linked
        if not self.runtime_library_dirs:
            self.runtime_library_dirs.append(get_config_var('LIBDIR'))
        if not self.libraries:
            libraries = ["python" + get_config_var("LDVERSION")]
        # if sys.platform != 'darwin':
        #     extra_postargs.append("-no-pie")
        if not extra_preargs:
            extra_preargs = []

        return super().link(target_desc, objects,
                            output_filename, output_dir, libraries,
                            library_dirs, runtime_library_dirs,
                            export_symbols, debug, extra_preargs,
                            extra_postargs, build_temp, target_lang)

    def runtime_library_dir_option(self, dir):
        if sys.platform == "darwin":
            return "-L" + dir
        else:
            return "-Wl,-R" + dir



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import io
import os
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import relpath
from os.path import splitext

from setuptools import Extension
from setuptools import find_packages
from setuptools import setup
from setuptools.command.build_ext import build_ext
from distutils.sysconfig import customize_compiler
from nasmcompiler import NasmCompiler


class NasmBuildCommand(build_ext):
    description = "build NASM extensions (compile/link to build directory)"

    def run(self):
        try:
            if self.distribution.has_c_libraries():
                build_clib = self.get_finalized_command('build_clib')
                self.libraries.extend(build_clib.get_library_names() or [])
                self.library_dirs.append(build_clib.build_clib)
            self.compiler = NasmCompiler(verbose=True)
            customize_compiler(self.compiler)

            # incase someone tries.
            if os.name == 'nt':
                raise NotImplementedError("Windows support not implemented for NASM")

            if self.include_dirs is not None:
                self.compiler.set_include_dirs(self.include_dirs)

            if self.define is not None:
                # 'define' option is a list of (name,value) tuples
                for (name, value) in self.define:
                    self.compiler.define_macro(name, value)
            if self.undef is not None:
                for macro in self.undef:
                    self.compiler.undefine_macro(macro)
            if self.libraries is not None:
                self.compiler.set_libraries(self.libraries)
            if self.library_dirs is not None:
                self.compiler.set_library_dirs(self.library_dirs)
            if self.rpath is not None:
                self.compiler.set_runtime_library_dirs(self.rpath)
            if self.link_objects is not None:
                self.compiler.set_link_objects(self.link_objects)

            self.build_extensions()
        except Exception as e:
            self._unavailable(e)
            self.extensions = []  # avoid copying missing files (it would fail).

    def _unavailable(self, e):
        print('*' * 80)
        print('''WARNING:

    An optional code optimization (C extension) could not be compiled.

    Optimizations for this package will not be available!
        ''')

        print('CAUSE:')
        print('')
        print('    ' + repr(e))
        print('*' * 80)


def read(*names, **kwargs):
    with io.open(
            join(dirname(__file__), *names),
            encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()


setup(
    name='pymult',
    version='0.0.2',
    license='BSD-2-Clause',
    description='An example Python package written in x86-64 assembly.',
    long_description= read('README.md'),
    author='Anthony Shaw',
    author_email='anthonyshaw@apache.org',
    url='https://github.com/tonybaloney/python-assembly-poc',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
    python_requires='>=3.7',
    install_requires=[
    ],
    extras_require={
    },
    cmdclass={'build_ext': NasmBuildCommand},
    ext_modules=[
        Extension(
            splitext(relpath(path, 'src').replace(os.sep, '.'))[0],
            sources=[path],
            extra_compile_args=[],
            extra_link_args=[],
            include_dirs=[dirname(path)]
        )
        for root, _, _ in os.walk('src')
        for path in glob(join(root, '*.asm'))
    ],
)
